@@ -1,5 +1,5 @@
 /* This file is part of the YAZ toolkit.
- * Copyright (C) 1995-2008 Index Data
+ * Copyright (C) 1995-2010 Index Data
  * See the file LICENSE for details.
  */
 
@@ -15,17 +15,18 @@
 #include <string.h>
 #include <yaz/yaz-version.h>
 
-unsigned long yaz_version(char *version_str, char *sys_str)
+unsigned long yaz_version(char *version_str, char *sha1_str)
 {
     if (version_str)
         strcpy(version_str, YAZ_VERSION);
-    if (sys_str)
-        strcpy(sys_str, "");
+    if (sha1_str)
+        strcpy(sha1_str, YAZ_VERSION_SHA1);
     return YAZ_VERSIONL;
 }
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
